@@ -23,6 +23,39 @@ logger = logging.getLogger("openassessment.assessment.api.peer")  # pylint: disa
 
 PEER_TYPE = "PE"
 
+FLEXIBLE_PEER_GRADING_REQUIRED_SUBMISSION_AGE_IN_DAYS = 7
+FLEXIBLE_PEER_GRADING_GRADED_BY_PERCENTAGE = 30
+
+
+def required_peer_grades(submission_uuid, peer_requirements):
+    """
+    Given a submission id, finds how many peer assessment required.
+
+    Args:
+        submission_uuid (str): The UUID of the submission being tracked.
+        peer_requirements (dict): Dictionary with the key "must_grade" indicating
+            the required number of submissions the student must grade
+            and "enable_flexible_grading" indicating if flexible grading enabled.
+
+    Returns:
+        int
+    """
+
+    submission = sub_api.get_submission(submission_uuid)
+
+    must_grade = peer_requirements["must_be_graded_by"]
+
+    if peer_requirements.get("enable_flexible_grading"):
+
+        # find how many days elapsed since subimitted
+        days_elapsed = (timezone.now().date() - submission['submitted_at'].date()).days
+
+        # check if flexible grading applies. if it does, then update must_grade
+        if days_elapsed >= FLEXIBLE_PEER_GRADING_REQUIRED_SUBMISSION_AGE_IN_DAYS:
+            must_grade = int(must_grade * FLEXIBLE_PEER_GRADING_GRADED_BY_PERCENTAGE / 100)
+
+    return must_grade
+
 
 def can_be_skipped(submission_uuid, peer_requirements):  # pylint: disable=unused-argument
     """
@@ -70,8 +103,8 @@ def submitter_is_finished(submission_uuid, peer_requirements):
         return False
     except PeerWorkflow.DoesNotExist:
         return False
-    except KeyError:
-        raise PeerAssessmentRequestError(u'Requirements dict must contain "must_grade" key')
+    except KeyError as ex:
+        raise PeerAssessmentRequestError('Requirements dict must contain "must_grade" key') from ex
 
 
 def assessment_is_finished(submission_uuid, peer_requirements):
@@ -104,7 +137,7 @@ def assessment_is_finished(submission_uuid, peer_requirements):
         assessment__submission_uuid=submission_uuid,
         assessment__score_type=PEER_TYPE
     )
-    return scored_items.count() >= peer_requirements["must_be_graded_by"]
+    return scored_items.count() >= required_peer_grades(submission_uuid, peer_requirements)
 
 
 def on_start(submission_uuid):
@@ -139,14 +172,14 @@ def on_start(submission_uuid):
         # If we get an integrity error, it means someone else has already
         # created a workflow for this submission, so we don't need to do anything.
         pass
-    except DatabaseError:
+    except DatabaseError as ex:
         error_message = (
-            u"An internal error occurred while creating a new peer "
-            u"workflow for submission {}"
+            "An internal error occurred while creating a new peer "
+            "workflow for submission {}"
             .format(submission_uuid)
         )
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
 
 
 def get_score(submission_uuid, peer_requirements):
@@ -164,6 +197,7 @@ def get_score(submission_uuid, peer_requirements):
         contributing_assessments information, along with a None staff_id.
 
     """
+
     if peer_requirements is None:
         return None
 
@@ -183,8 +217,8 @@ def get_score(submission_uuid, peer_requirements):
         assessment__score_type=PEER_TYPE
     ).order_by('-assessment')
 
-    submission_finished = items.count() >= peer_requirements["must_be_graded_by"]
-    if not submission_finished:
+    # Check if enough peers have graded this submission
+    if items.count() < required_peer_grades(submission_uuid, peer_requirements):
         return None
 
     # Unfortunately, we cannot use update() after taking a slice,
@@ -272,8 +306,8 @@ def create_assessment(
         peer_workflow_item = scorer_workflow.find_active_assessments()
         if peer_workflow_item is None:
             message = (
-                u"There are no open assessments associated with the scorer's "
-                u"submission UUID {}."
+                "There are no open assessments associated with the scorer's "
+                "submission UUID {}."
             ).format(scorer_submission_uuid)
             logger.warning(message)
             raise PeerAssessmentWorkflowError(message)
@@ -293,27 +327,27 @@ def create_assessment(
 
         _log_assessment(assessment, scorer_workflow)
         return full_assessment_dict(assessment)
-    except PeerWorkflow.DoesNotExist:
+    except PeerWorkflow.DoesNotExist as ex:
         message = (
-            u"There is no Peer Workflow associated with the given "
-            u"submission UUID {}."
+            "There is no Peer Workflow associated with the given "
+            "submission UUID {}."
         ).format(scorer_submission_uuid)
         logger.exception(message)
-        raise PeerAssessmentWorkflowError(message)
-    except InvalidRubric:
-        msg = u"The rubric definition is not valid."
+        raise PeerAssessmentWorkflowError(message) from ex
+    except InvalidRubric as ex:
+        msg = "The rubric definition is not valid."
         logger.exception(msg)
-        raise PeerAssessmentRequestError(msg)
-    except InvalidRubricSelection:
-        msg = u"Invalid options were selected in the rubric."
+        raise PeerAssessmentRequestError(msg) from ex
+    except InvalidRubricSelection as ex:
+        msg = "Invalid options were selected in the rubric."
         logger.warning(msg, exc_info=True)
-        raise PeerAssessmentRequestError(msg)
-    except DatabaseError:
+        raise PeerAssessmentRequestError(msg) from ex
+    except DatabaseError as ex:
         error_message = (
-            u"An error occurred while creating an assessment by the scorer with this ID: {}"
+            "An error occurred while creating an assessment by the scorer with this ID: {}"
         ).format(scorer_id)
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
 
 
 @transaction.atomic
@@ -412,12 +446,12 @@ def get_rubric_max_scores(submission_uuid):
             criterion["name"]: criterion["points_possible"]
             for criterion in rubric_dict["criteria"]
         }
-    except DatabaseError:
+    except DatabaseError as ex:
         error_message = (
-            u"Error getting rubric options max scores for submission uuid {uuid}"
+            "Error getting rubric options max scores for submission uuid {uuid}"
         ).format(uuid=submission_uuid)
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
 
 
 def get_assessment_median_scores(submission_uuid):
@@ -452,12 +486,12 @@ def get_assessment_median_scores(submission_uuid):
         return Assessment.get_median_score_dict(scores)
     except PeerWorkflow.DoesNotExist:
         return {}
-    except DatabaseError:
+    except DatabaseError as ex:
         error_message = (
-            u"Error getting assessment median scores for submission {uuid}"
+            "Error getting assessment median scores for submission {uuid}"
         ).format(uuid=submission_uuid)
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
 
 
 def has_finished_required_evaluating(submission_uuid, required_assessments):
@@ -552,12 +586,12 @@ def get_assessments(submission_uuid, limit=None):
             score_type=PEER_TYPE
         )[:limit]
         return serialize_assessments(assessments)
-    except DatabaseError:
+    except DatabaseError as ex:
         error_message = (
-            u"Error getting assessments for submission {uuid}"
+            "Error getting assessments for submission {uuid}"
         ).format(uuid=submission_uuid)
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
 
 
 def get_submitted_assessments(submission_uuid, limit=None):
@@ -618,12 +652,12 @@ def get_submitted_assessments(submission_uuid, limit=None):
         assessments = Assessment.objects.filter(
             pk__in=[item.assessment.pk for item in items])[:limit]
         return serialize_assessments(assessments)
-    except DatabaseError:
+    except DatabaseError as ex:
         error_message = (
-            u"Couldn't retrieve the assessments completed by the student with submission {uuid}"
+            "Couldn't retrieve the assessments completed by the student with submission {uuid}"
         ).format(uuid=submission_uuid)
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
 
 
 def get_submission_to_assess(submission_uuid, graded_by):
@@ -674,8 +708,8 @@ def get_submission_to_assess(submission_uuid, graded_by):
 
     if not workflow:
         raise PeerAssessmentWorkflowError(
-            u"A Peer Assessment Workflow does not exist for the student "
-            u"with submission UUID {}".format(submission_uuid)
+            "A Peer Assessment Workflow does not exist for the student "
+            "with submission UUID {}".format(submission_uuid)
         )
 
     if workflow.is_cancelled:
@@ -696,16 +730,16 @@ def get_submission_to_assess(submission_uuid, graded_by):
             PeerWorkflow.create_item(workflow, peer_submission_uuid)
             _log_workflow(peer_submission_uuid, workflow)
             return submission_data
-        except sub_api.SubmissionNotFoundError:
+        except sub_api.SubmissionNotFoundError as ex:
             error_message = (
-                u"Could not find a submission with the uuid {} for student {} "
-                u"in the peer workflow."
+                "Could not find a submission with the uuid {} for student {} "
+                "in the peer workflow."
             ).format(peer_submission_uuid, workflow.student_id)
             logger.exception(error_message)
-            raise PeerAssessmentWorkflowError(error_message)
+            raise PeerAssessmentWorkflowError(error_message) from ex
     else:
         logger.info(
-            u"No submission found for {} to assess ({}, {})"
+            "No submission found for {} to assess ({}, {})"
             .format(
                 workflow.student_id,
                 workflow.course_id,
@@ -750,13 +784,13 @@ def create_peer_workflow(submission_uuid):
         # If we get an integrity error, it means someone else has already
         # created a workflow for this submission, so we don't need to do anything.
         pass
-    except DatabaseError:
+    except DatabaseError as ex:
         error_message = (
-            u"An internal error occurred while creating a new peer "
-            u"workflow for submission {}"
+            "An internal error occurred while creating a new peer "
+            "workflow for submission {}"
         ).format(submission_uuid)
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
 
 
 def create_peer_workflow_item(scorer_submission_uuid, submission_uuid):
@@ -800,13 +834,13 @@ def get_assessment_feedback(submission_uuid):
         return AssessmentFeedbackSerializer(feedback).data
     except AssessmentFeedback.DoesNotExist:
         return None
-    except DatabaseError:
+    except DatabaseError as ex:
         error_message = (
-            u"An error occurred retrieving assessment feedback for {}."
+            "An error occurred retrieving assessment feedback for {}."
             .format(submission_uuid)
         )
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
 
 
 def set_assessment_feedback(feedback_dict):
@@ -831,7 +865,7 @@ def set_assessment_feedback(feedback_dict):
     selected_options = feedback_dict.get('options', list())
 
     if feedback_text and len(feedback_text) > AssessmentFeedback.MAXSIZE:
-        error_message = u"Assessment feedback too large."
+        error_message = "Assessment feedback too large."
         raise PeerAssessmentRequestError(error_message)
 
     try:
@@ -841,7 +875,7 @@ def set_assessment_feedback(feedback_dict):
         if submission_uuid:
             feedback, created = AssessmentFeedback.objects.get_or_create(submission_uuid=submission_uuid)
         else:
-            error_message = u"An error occurred creating assessment feedback: bad or missing submission_uuid."
+            error_message = "An error occurred creating assessment feedback: bad or missing submission_uuid."
             logger.error(error_message)
             raise PeerAssessmentRequestError(error_message)
 
@@ -859,10 +893,10 @@ def set_assessment_feedback(feedback_dict):
         # Associate the feedback with scored assessments
         assessments = PeerWorkflowItem.get_scored_assessments(submission_uuid)
         feedback.assessments.add(*assessments)
-    except DatabaseError:
-        msg = u"Error occurred while creating or updating feedback on assessment: {}".format(feedback_dict)
+    except DatabaseError as ex:
+        msg = f"Error occurred while creating or updating feedback on assessment: {feedback_dict}"
         logger.exception(msg)
-        raise PeerAssessmentInternalError(msg)
+        raise PeerAssessmentInternalError(msg) from ex
 
 
 def _log_assessment(assessment, scorer_workflow):
@@ -879,9 +913,9 @@ def _log_assessment(assessment, scorer_workflow):
 
     """
     logger.info(
-        u"Created peer-assessment {assessment_id} for submission "
-        u"{submission_uuid}, course {course_id}, item {item_id} "
-        u"with rubric {rubric_content_hash}; scored by {scorer}"
+        "Created peer-assessment {assessment_id} for submission "
+        "{submission_uuid}, course {course_id}, item {item_id} "
+        "with rubric {rubric_content_hash}; scored by {scorer}"
         .format(
             assessment_id=assessment.id,
             submission_uuid=assessment.submission_uuid,
@@ -903,7 +937,7 @@ def _log_workflow(submission_uuid, workflow):
             assessment.
     """
     logger.info(
-        u"Retrieved submission {} ({}, {}) to be assessed by {}"
+        "Retrieved submission {} ({}, {}) to be assessed by {}"
         .format(
             submission_uuid,
             workflow.course_id,
@@ -949,11 +983,11 @@ def on_cancel(submission_uuid):
         if workflow:
             workflow.cancelled_at = timezone.now()
             workflow.save()
-    except (PeerAssessmentWorkflowError, DatabaseError):
+    except (PeerAssessmentWorkflowError, DatabaseError) as ex:
         error_message = (
-            u"An internal error occurred while cancelling the peer"
-            u"workflow for submission {}"
+            "An internal error occurred while cancelling the peer"
+            "workflow for submission {}"
             .format(submission_uuid)
         )
         logger.exception(error_message)
-        raise PeerAssessmentInternalError(error_message)
+        raise PeerAssessmentInternalError(error_message) from ex
