@@ -54,7 +54,7 @@ class ConfigMixin:
         with the given ``switch_name``.
         """
         # pylint: disable=feature-toggle-needs-doc
-        return WaffleSwitch("{}.{}".format(WAFFLE_NAMESPACE, switch_name), module_name=__name__)
+        return WaffleSwitch(f"{WAFFLE_NAMESPACE}.{switch_name}", module_name=__name__)
 
     @staticmethod
     def _course_waffle_flag(flag_name):
@@ -63,7 +63,8 @@ class ConfigMixin:
         with the given ``flag_name``.
         """
         CourseWaffleFlag = import_course_waffle_flag()  # pylint: disable=invalid-name
-        return CourseWaffleFlag(WAFFLE_NAMESPACE, flag_name)  # pylint: disable=feature-toggle-needs-doc
+        # pylint: disable=feature-toggle-needs-doc
+        return CourseWaffleFlag(WAFFLE_NAMESPACE, flag_name, module_name=__name__)
 
     @staticmethod
     def _settings_toggle_enabled(toggle_name):
