@@ -35,8 +35,8 @@ def utf8_validator(value):
         if isinstance(value, bytes):
             return value.decode('utf-8')
         return str(value)
-    except (ValueError, TypeError):
-        raise Invalid(u"Could not load unicode from value \"{val}\"".format(val=value))
+    except (ValueError, TypeError) as ex:
+        raise Invalid(f"Could not load unicode from value \"{value}\"") from ex
 
 
 def datetime_validator(value):
@@ -56,39 +56,39 @@ def datetime_validator(value):
         # The dateutil parser defaults empty values to the current day,
         # which is NOT what we want.
         if value is None or value == '':
-            raise Invalid(u"Datetime value cannot be \"{val}\"".format(val=value))
+            raise Invalid(f"Datetime value cannot be \"{value}\"")
 
         # Parse the date and interpret it as UTC
         value = dateutil.parser.parse(value).replace(tzinfo=utc)
         return str(value.isoformat())
-    except (ValueError, TypeError):
-        raise Invalid(u"Could not parse datetime from value \"{val}\"".format(val=value))
+    except (ValueError, TypeError) as ex:
+        raise Invalid(f"Could not parse datetime from value \"{value}\"") from ex
 
 
 PROMPTS_TYPES = [
-    u'text',
-    u'html',
+    'text',
+    'html',
 ]
 
 
 NECESSITY_OPTIONS = [
-    u'required',
-    u'optional',
-    u''
+    'required',
+    'optional',
+    ''
 ]
 
 
 VALID_ASSESSMENT_TYPES = [
-    u'peer-assessment',
-    u'self-assessment',
-    u'student-training',
-    u'staff-assessment',
+    'peer-assessment',
+    'self-assessment',
+    'student-training',
+    'staff-assessment',
 ]
 
 VALID_UPLOAD_FILE_TYPES = [
-    u'image',
-    u'pdf-and-image',
-    u'custom'
+    'image',
+    'pdf-and-image',
+    'custom'
 ]
 
 # Schema definition for an update from the Studio JavaScript editor.
@@ -122,6 +122,7 @@ EDITOR_UPDATE_SCHEMA = Schema({
             'required': bool,
             'must_grade': All(int, Range(min=0)),
             'must_be_graded_by': All(int, Range(min=0)),
+            Required('enable_flexible_grading', default=False): bool,
             'examples': [
                 Schema({
                     Required('answer'): [utf8_validator],
@@ -139,7 +140,7 @@ EDITOR_UPDATE_SCHEMA = Schema({
     Required('editor_assessments_order'): [
         All(utf8_validator, In(VALID_ASSESSMENT_TYPES))
     ],
-    Required('feedbackprompt', default=u""): utf8_validator,
+    Required('feedbackprompt', default=""): utf8_validator,
     Required('criteria'): [
         Schema({
             Required('order_num'): All(int, Range(min=0)),
