@@ -208,13 +208,16 @@ class StudioMixin:
                 return {'success': False, 'msg': self._('Error updating XBlock configuration')}
 
         if not data['text_response'] and not data['file_upload_response']:
-            return {'success': False, 'msg': self._("Error: both text and file upload responses can't be disabled")}
+            return {
+                'success': False,
+                'msg': self._("Error: Text Response and File Upload Response cannot both be disabled")
+            }
         if not data['text_response'] and data['file_upload_response'] == 'optional':
             return {'success': False,
-                    'msg': self._("Error: in case if text response is disabled file upload response must be required")}
+                    'msg': self._("Error: When Text Response is disabled, File Upload Response must be Required")}
         if not data['file_upload_response'] and data['text_response'] == 'optional':
             return {'success': False,
-                    'msg': self._("Error: in case if file upload response is disabled text response must be required")}
+                    'msg': self._("Error: When File Upload Response is disabled, Text Response must be Required")}
 
         # Backwards compatibility: We used to treat "name" as both a user-facing label
         # and a unique identifier for criteria and options.
@@ -238,7 +241,7 @@ class StudioMixin:
             leaderboard_show=data['leaderboard_show']
         )
         if not success:
-            return {'success': False, 'msg': self._(u'Validation error: {error}').format(error=msg)}
+            return {'success': False, 'msg': self._('Validation error: {error}').format(error=msg)}
 
         # At this point, all the input data has been validated,
         # so we can safely modify the XBlock fields.
@@ -267,7 +270,7 @@ class StudioMixin:
         self.teams_enabled = bool(data.get('teams_enabled', False))
         self.selected_teamset_id = data.get('selected_teamset_id', '')
 
-        return {'success': True, 'msg': self._(u'Successfully updated OpenAssessment XBlock')}
+        return {'success': True, 'msg': self._('Successfully updated OpenAssessment XBlock')}
 
     @XBlock.json_handler
     def check_released(self, data, suffix=''):  # pylint: disable=unused-argument
@@ -286,7 +289,7 @@ class StudioMixin:
         # There aren't currently any server-side error conditions we report to the client,
         # but we send success/msg values anyway for consistency with other handlers.
         return {
-            'success': True, 'msg': u'',
+            'success': True, 'msg': '',
             'is_released': self.is_released()
         }
 
